/* Title: Asynchronous example */

#include <err.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

int
main(int argc, char *argv[])
{
	int	 c, async = 0;
	pid_t	 pid;

	/* 
	 * Catch whether we're reentering.
	 * This is "hackish", but works very well.
	 */

	while (-1 != (c = getopt(argc, argv, "X"))) 
		switch (c) {
		case 'X':
			async = 1;
			break;
		default:
			return(EXIT_SUCCESS);
		}

	if (async) {	
		/*
		 * Here we'd do our hard work.
		 * We'll just sleep for a long time.
		 */
		sleep(20);
		return(EXIT_SUCCESS);
	}

	/* Give our response to the server. */

	puts("Status: 200 OK\r");
	puts("Content-Type: text/html\r");
	puts("\r");
	puts("Hello, world!");

	/* Start our worker child. */

	if (0 == (pid = fork())) {
		/*
		 * Daemonise to close our file descriptors.
		 * This will often "release" the calling web 
		 * server from our request.
		 */
		if (-1 == daemon(1, 0))
			err(EXIT_FAILURE, NULL);
		execl(argv[0], argv[0], "-X", (char *)NULL);
		err(EXIT_FAILURE, "%s", argv[0]);
	} else if (-1 == pid)
		err(EXIT_FAILURE, NULL);

	/* 
	 * Parent exits now, so if our web server is
	 * waiting for child exit to release the request,
	 * it will do so now.
	 */

	return(EXIT_SUCCESS);
}
